@@ -1,7 +1,7 @@
 """This module contains constants used by other modules."""
-MAJOR_VERSION = 3
-MINOR_VERSION = 5
-PATCH_VERSION = 2
+MAJOR_VERSION = 4
+MINOR_VERSION = 1
+PATCH_VERSION = 0
 __short_version__ = "{}.{}".format(MAJOR_VERSION, MINOR_VERSION)
 __version__ = "{}.{}".format(__short_version__, PATCH_VERSION)
 REQUIRED_PYTHON_VER = (3, 6)
@@ -12,7 +12,7 @@ BASE_UUID = "-0000-1000-8000-0026BB765291"
 STANDALONE_AID = 1  # Standalone accessory ID (i.e. not bridged)
 
 # ### Default values ###
-DEFAULT_CONFIG_VERSION = 2
+DEFAULT_CONFIG_VERSION = 1
 DEFAULT_PORT = 51827
 
 # ### Configuration version ###
@@ -69,14 +69,19 @@ HAP_REPR_FORMAT = "format"
 HAP_REPR_IID = "iid"
 HAP_REPR_MAX_LEN = "maxLen"
 HAP_REPR_PERM = "perms"
+HAP_REPR_PID = "pid"
 HAP_REPR_PRIMARY = "primary"
 HAP_REPR_SERVICES = "services"
 HAP_REPR_LINKED = "linked"
 HAP_REPR_STATUS = "status"
+HAP_REPR_TTL = "ttl"
 HAP_REPR_TYPE = "type"
 HAP_REPR_VALUE = "value"
 HAP_REPR_VALID_VALUES = "valid-values"
 
+HAP_PROTOCOL_VERSION = "01.01.00"
+HAP_PROTOCOL_SHORT_VERSION = "1.1"
+
 
 # Status codes for underlying HAP calls
 class HAP_SERVER_STATUS:
@@ -92,3 +97,12 @@ class HAP_SERVER_STATUS:
     RESOURCE_DOES_NOT_EXIST = -70409
     INVALID_VALUE_IN_REQUEST = -70410
     INSUFFICIENT_AUTHORIZATION = -70411
+
+
+class HAP_PERMISSIONS:
+    USER = b"\x00"
+    ADMIN = b"\x01"
+
+
+# Client properties
+CLIENT_PROP_PERMS = "permissions"
